unit Beveles;
(***********************************************************************
 *                                                                     *
 *  Copyright (C) 1996-1997  Erol S. Uzuner                            *
 *                                                                     *
 *  Modul       :       Bevels                                         *
 *  Version     :       1.0 (9.9.96)                                   *
 *                                                                     *
 *  Beschreibung:       TBevel mit erweiterten optionen                *
 *                                                                     *
 *                                                                     *
 *  Autor       :       Erol S. Uzuner                                 *
 *                                                                     *
 *  Datum       :       Mon Okt 9  20:00:00 1996                       *
 *                                                                     *
 *                                                                     *
 *  Aenderungen :                                                      *
 *  Datum        Autor          Beschreibung                           *
 *  28.08.97     Erol S. Uzuner  berarbeitung                         *
 *                                                                     *
 *                                                                     *
 ***********************************************************************)
interface

uses
  SysUtils, WinTypes,  Classes, Graphics, Controls,
  ExtCtrls;

type


  TBeveles = class(TBevel)
  private
    { Private-Deklarationen }
    FBorderWidth : Byte;
    FStyle: TBevelStyle;
    FCaption : TCaption;
    procedure SetStyle(Value: TBevelStyle);
    procedure SetCaption(Value : TCaption);
    procedure SetBorderWidth(Value : Byte);
  protected
    { Protected-Deklarationen }
    procedure Paint; override;
  public
    { Public-Deklarationen }
    constructor Create(AOwner: TComponent); override;
  published
    { Published-Deklarationen }
    property Style: TBevelStyle read FStyle write SetStyle default bsLowered;
    property Caption: TCaption read FCaption write SetCaption;
    property Font;
    property BorderWidth : Byte read FBorderWidth write SetBorderWidth;
    property Shape;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('sarcon', [TBeveles]);
end;

constructor TBeveles.Create(AOwner : TComponent);
begin
    inherited Create(AOwner);
    FBorderWidth := 0;
    Text := '';
end;

procedure TBeveles.Paint;
var
  H: Integer;
  R: TRect;
  C: array [Byte] of Char;
  CLen: Integer;
  Color1, Color2, Temp : TColor;
  CaptionExists : Boolean;


  procedure BevelLine(C: TColor; X1, Y1, X2, Y2: Integer);
  begin
    with Canvas do
    begin
      Pen.Color := C;
      MoveTo(X1, Y1);
      LineTo(X2, Y2);
    end;
  end;

  procedure DrawBevelText;
  begin
       with Canvas do
       begin
           StrPCopy(C, Text);
           if C[0] <> #0 then
           begin
             StrPCopy(C, Text);
             CLen := StrLen(C);
             R := Rect(8, 0, 0, H);
             DrawText(Canvas.Handle, C, CLen, R, DT_LEFT or DT_SINGLELINE or DT_CALCRECT);
             Brush.Style := bsClear;
             DrawText(Canvas.Handle, C, CLen, R, DT_LEFT or DT_SINGLELINE);
           end;
       end;
  end;

  procedure BevelBox;
  begin
     with Canvas do
     begin
          Pen.Color := Color1;

          if CaptionExists then begin
              PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
                        Point(R.Left + 6, R.Top)]);
              PolyLine([Point(R.Left + TextWidth(Self.Text)+ 12, R.Top), Point(R.Right,R.Top)])
           end else begin
              PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
                        Point(R.Left, R.Top)]);
              PolyLine([Point(R.Left, R.Top), Point(R.Right,R.Top)])
           end;

           Dec(R.Bottom);
           Dec(R.Right);
           Pen.Color := Color2;
           PolyLine([Point(R.Right, R.Top), Point(R.Right, R.Bottom),
                     Point(R.Left, R.Bottom)]);
     end;
  end;

  procedure BevelFrame;
  begin
     with Canvas do
     begin
          Pen.Color := Color1;

          Dec(R.Bottom,2);
          Dec(R.Right);
          if CaptionExists then begin
            PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
                     Point(R.Left + 6, R.Top)]);
            PolyLine([Point(R.Left + TextWidth(Self.Text)+ 6, R.Top), Point(R.Right,R.Top)]);
          end else begin
            PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
                     Point(R.Left, R.Top)]);
            PolyLine([Point(R.Left, R.Top), Point(R.Right,R.Top)]);
          end;

          Dec(R.Right);
          PolyLine([Point(R.Right, R.Top), Point(R.Right, R.Bottom),
                     Point(R.Left, R.Bottom)]);

          Pen.Color := Color2;
          Inc(R.Left);
          Inc(R.Top);
          Dec(R.Bottom);
          Dec(R.Right);
          if CaptionExists then begin
            PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
                       Point(R.Left + 6, R.Top)]);
            PolyLine([Point(R.Left + TextWidth(Self.Text)+ 6, R.Top), Point(R.Right,R.Top)]);
          end else begin
            PolyLine([Point(R.Left, R.Bottom), Point(R.Left, R.Top),
                       Point(R.Left, R.Top)]);
            PolyLine([Point(R.Left, R.Top), Point(R.Right,R.Top)]);
          end;

          Dec(R.Left);
          Inc(R.Bottom,2);
          Inc(R.Right,2);
          PolyLine([Point(R.Right, R.Top), Point(R.Right, R.Bottom),
                     Point(R.Left, R.Bottom)]);
     end;
  end;

  procedure BevelInnerBox;
  begin
       if FBorderWidth > 0 then
       begin
         with canvas do
         begin
             Temp := Color1;
             Color1 := Color2;
             Color2 := Temp;
             Inc(R.Left, FBorderWidth);
             Inc(R.Top, FBorderWidth);
             Dec(R.Right, FBorderWidth);
             Dec(R.Bottom, FBorderWidth);
             BevelBox;
          end;
       end;
  end;

begin
  if Caption <> '' then CaptionExists := true
  else CaptionExists := false;

  with Canvas do
  begin
    Pen.Width := 1;
    if CaptionExists then begin
      Font := Self.Font;
      H := TextHeight('0');
      R := Rect(0, (H Div 2) - 1 , Width, Height);
    end else
      R := Rect(0, 0, Width, Height);

    if FStyle = bsLowered then
    begin
      Color1 := clBtnShadow;
      Color2 := clBtnHighlight;
    end
    else
    begin
      Color1 := clBtnHighlight;
      Color2 := clBtnShadow;
    end;

    case Shape of
      bsBox:
        begin
           BevelBox;
           Inc(R.Bottom);
           Inc(R.Right);
           BevelInnerBox;
           if CaptionExists then DrawBevelText;
        end;
      bsFrame:
        begin
          BevelFrame;
          Inc(R.Left);
          BevelInnerBox;
          if CaptionExists then DrawBevelText;
        end;
      bsTopLine:
        begin
          BevelLine(Color1, 0, 0, Width, 0);
          BevelLine(Color2, 0, 1, Width, 1);
          if CaptionExists then DrawBevelText;
        end;
      bsBottomLine:
        begin
          BevelLine(Color1, 0, Height - 2, Width, Height - 2);
          BevelLine(Color2, 0, Height - 1, Width, Height - 1);
          if CaptionExists then DrawBevelText;
        end;
      bsLeftLine:
        begin
          BevelLine(Color1, 0, 0, 0, Height);
          BevelLine(Color2, 1, 0, 1, Height);
          if CaptionExists then DrawBevelText;
        end;
      bsRightLine:
        begin
          BevelLine(Color1, Width - 2, 0, Width - 2, Height);
          BevelLine(Color2, Width - 1, 0, Width - 1, Height);
          if CaptionExists then DrawBevelText;
        end;
    end;
  end;
end;

procedure TBeveles.SetStyle(Value: TBevelStyle);
begin
  if Value <> FStyle then
  begin
    FStyle := Value;
    Invalidate;
  end;
end;

procedure TBeveles.SetCaption(Value: TCaption);
begin
    if FCaption <> Value then
    begin
      FCaption := Value;
      Text := Value;
      Invalidate;
    end;
end;

procedure TBeveles.SetBorderWidth(Value : Byte);
begin
    if FBorderWidth <> Value then
    begin
       FBorderWidth := Value;
       Invalidate;
    end;
end;

end.
